#include <iostream>
#include <stdio.h>
#include <stdlib.h>
#include <rw/thread/RWThreadFunction.h>
#include <rw/thread/rwtMakeThreadFunction.h>
#include <rw/sync/RWSemaphore.h> 
using namespace std;


RWSemaphore ready(0);
RWSemaphore go(0);


#define OTL_ODBC
#define OTL_ODBC_UNIX
#include <otlv4.h>

void insert(int threadID, int records, int offset)
{
  otl_connect connection;
  try
  {
    connection.rlogon("UID=szis;PWD=;DSN=PostgreSQL");
  }
  catch( otl_exception &exp )
  {
    cout << "Thread [" << threadID << "] Error: Cannot connect:" << endl;
    cout << exp.msg << endl;
    cout << exp.stm_text << endl;
    cout << exp.sqlstate << endl;
    cout << exp.var_info << endl;
    connection.logoff();
    return;
  }

	ready.release(); // notify main that thread is ready to insert
	go.acquire();    // wait for all theads to reach that state

  try
  {
    otl_stream os(1, "insert into test values(:num<int>, :str<char[50]>)",
                  connection);
    os.set_commit(0); // do not auto-commit on the stream
    for(int i = 0; i < records; i++)
    {
      char str[50];
      int num = i + offset;
      sprintf(str, "String %d", num);
      os << num << str;
      connection.commit();
    }
  }
  catch( otl_exception &exp )
  {
    connection.rollback();
		cout << "Thread [" << threadID << "] Caught otl_exception:" << endl;
    cout << exp.msg << endl;
    cout << exp.stm_text << endl;
    cout << exp.sqlstate << endl;
    cout << exp.var_info << endl;
  }
  connection.logoff();
}

int main(int argc, char **argv)
{
  if ( argc != 3 )
  {
		cout << "Inserts a specified number of records into the test table" << endl;
 		cout << "Usage: insert <num of alarms> <num of threads>" << endl;   
		return 1;
	}

	int num_alarms = atoi(argv[1]);
	int num_threads = atoi(argv[2]);
	
	if( num_alarms <= 0 )
	{
		cout << "The number of alarms must be greater than 0" << endl;
		return 1;
	}
	if( num_threads <= 0 )
	{
		cout << "The number of threads must be greater than 0" << endl;
		return 1;
	}

  otl_connect::otl_initialize( 1 ); // 1 = multithreaded program

	RWThreadFunction *threads = new RWThreadFunction[num_threads];
	for( int i = 0; i < num_threads; i++ )
	{
		int offset = i * num_alarms;
		threads[i] = rwtMakeThreadFunction(insert, i, num_alarms, offset);
	}

	for( int i = 0; i < num_threads; i++ )
		threads[i].start();

	for( int i = 0; i < num_threads; i++ )
		ready.acquire();

	for( int i = 0; i < num_threads; i++ )
		go.release();

	for( int i = 0; i < num_threads; i++ )
		threads[i].join();

	delete [] threads;


  return 0;
}
  
